-module(alarm).
-export([start/2,cancel/1,alarm/3,sleep/1,flush/0,priority_rec/0]).

start(Time, Alarm) ->
    spawn(alarm, alarm, [self(),Time,Alarm]).

cancel(Timer) ->
    Timer ! {self(),cancel}.

alarm(Pid, Time, Alarm) ->
    receive
        {Pid,cancel} -> true
    % DO SOMETHING IF NO MESSAGE ARRIVES IN TIME
    after Time -> Pid ! Alarm
    % special values:
    % infinity: timeout will never occur 
    %           useful if time is computed at runtime
    % 0: tries all messages currently in the mailbox first
    end.

% c(alarm).
% alarm:start(10000,wake_up).
% receive A -> A end.
% 
% Al = alarm:start(30000,wake_up).
% alarm:cancel(Al).

%% EXAMPLES with timeouts

sleep(Time) ->
    receive
    after Time -> true
    end.
% alarm:sleep(5000).



flush() ->
    receive
        Any -> io:format("flush ~p~n",[Any]),
               flush()
    after 0 -> true
    end.
% self() ! one.
% self() ! two.
% self() ! three.
% self() ! four.
% alarm:flush().

priority_rec() ->
    receive
        {bad,News} -> News
    after 0 ->
        receive
            {good,News} -> News
        end
    end.

% self() ! {good,goal}.
% self() ! {good,win}. 
% self() ! {bad,loose}.
% self() ! {good,baby_born}.
% alarm:priority_rec(). 


